@@ -133,6 +133,56 @@ HOTKEYS = [
             },
         ]
     },
+    {
+        'label': _('Split management'),
+        'key': 'split',
+        'keys': [
+            {
+                'key': 'split-tab-vertical',
+                'label': _('Split tab vertical')
+            },
+            {
+                'key': 'split-tab-horizontal',
+                'label': _('Split tab horizontal')
+            },
+            {
+                'key': 'close-terminal',
+                'label': _('Close terminal')
+            },
+            {
+                'key': 'focus-terminal-up',
+                'label': _('Focus terminal above')
+            },
+            {
+                'key': 'focus-terminal-down',
+                'label': _('Focus terminal below')
+            },
+            {
+                'key': 'focus-terminal-left',
+                'label': _('Focus terminal on the left')
+            },
+            {
+                'key': 'focus-terminal-right',
+                'label': _('Focus terminal on the right')
+            },
+            {
+                'key': 'move-terminal-split-up',
+                'label': _('Move the terminal split handle up')
+            },
+            {
+                'key': 'move-terminal-split-down',
+                'label': _('Move the terminal split handle down')
+            },
+            {
+                'key': 'move-terminal-split-right',
+                'label': _('Move the terminal split handle right')
+            },
+            {
+                'key': 'move-terminal-split-left',
+                'label': _('Move the terminal split handle left')
+            },
+        ]
+    },
     {
         'label': _('Navigation'),
         'key': 'nav',
@@ -287,7 +337,7 @@ def refresh_user_start(settings):
             os.remove(desktop_file)
 
 
-class PrefsCallbacks(object):
+class PrefsCallbacks():
 
     """Holds callbacks that will be used in the PrefsDialg class.
     """
@@ -679,8 +729,9 @@ class PrefsDialog(SimpleGladeApp):
         treeview.set_model(self.store)
         treeview.set_rules_hint(True)
 
-        # TODO PORT this is killing the editing of the accl
-        # treeview.connect('button-press-event', self.start_editing)
+        treeview.connect('button-press-event', self.start_editing)
+
+        treeview.set_activate_on_single_click(True)
 
         renderer = Gtk.CellRendererText()
         column = Gtk.TreeViewColumn(_('Action'), renderer, text=1)
@@ -698,7 +749,13 @@ class PrefsDialog(SimpleGladeApp):
         column.add_attribute(renderer, "accel-key", 1)
         treeview.append_column(column)
 
-        self.demo_terminal = GuakeTerminal(self.settings)
+        class fake_guake():
+            pass
+
+        fg = fake_guake()
+        fg.window = self.window
+        fg.settings = self.settings
+        self.demo_terminal = GuakeTerminal(fg)
         self.demo_terminal_box = self.get_widget('demo_terminal_box')
         self.demo_terminal_box.add(self.demo_terminal)
 
@@ -944,7 +1001,6 @@ class PrefsDialog(SimpleGladeApp):
         if hook_show_widget is not None:
             if hook_show_setting is not None:
                 hook_show_widget.set_text(hook_show_setting)
-        return
 
     def _load_default_shell_settings(self):
         combo = self.get_widget('default_shell')
@@ -967,7 +1023,7 @@ class PrefsDialog(SimpleGladeApp):
         if dest_screen > n_screens - 1:
             self.settings.general.set_boolean('mouse-display', False)
             dest_screen = screen.get_primary_monitor()
-            self.settings.general.set_int('display_n', dest_screen)
+            self.settings.general.set_int('display-n', dest_screen)
 
         if dest_screen == ALWAYS_ON_PRIMARY:
             first_item = combo.get_model().get_iter_first()
@@ -1300,8 +1356,8 @@ class PrefsDialog(SimpleGladeApp):
 
         # avoiding problems with common keys
         if ((mods == 0 and key != 0) and
-            ((key >= ord('a') and key <= ord('z')) or (key >= ord('A') and key <= ord('Z')) or
-             (key >= ord('0') and key <= ord('9')))):
+            ((ord('a') <= key <= ord('z')) or (ord('A') <= key <= ord('Z')) or
+             (ord('0') <= key <= ord('9')))):
             dialog = Gtk.MessageDialog(
                 self.get_widget('config-window'),
                 Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
@@ -1321,7 +1377,7 @@ class PrefsDialog(SimpleGladeApp):
 
         self.store[path][HOTKET_MODEL_INDEX_HUMAN_ACCEL] = keylabel
 
-        if dconf_path == "show-hide" or dconf_path == "show-focus":
+        if dconf_path in ("show-hide", "show-focus"):
             self.settings.keybindingsGlobal.set_string(dconf_path, accelerator)
         else:
             self.settings.keybindingsLocal.set_string(dconf_path, accelerator)
@@ -1353,30 +1409,20 @@ class PrefsDialog(SimpleGladeApp):
 
         Thanks to gnome-keybinding-properties.c =)
         """
-        # TODO PORT some thing in here is breaking stuff
-
-        if event.window != treeview.get_bin_window():
-            return False
-
         x, y = int(event.x), int(event.y)
         ret = treeview.get_path_at_pos(x, y)
         if not ret:
             return False
 
         path, column, cellx, celly = ret
-        if path and len(path) > 1:
 
-            def real_cb():
-                treeview.grab_focus()
-                treeview.set_cursor(path, column, True)
+        treeview.row_activated(path, Gtk.TreeViewColumn(None))
+        treeview.set_cursor(path)
 
-            treeview.stop_emission('button-press-event')
-            GObject.idle_add(real_cb)
-
-        return True
+        return False
 
 
-class KeyEntry(object):
+class KeyEntry():
 
     def __init__(self, keycode, mask):
         self.keycode = keycode
