@@ -23,6 +23,7 @@ import os
 import re
 import signal
 import subprocess
+import sys
 import threading
 import uuid
 
@@ -50,55 +51,53 @@ from guake.globals import TERMINAL_MATCH_TAGS
 
 log = logging.getLogger(__name__)
 
+libutempter = None
+try:
+    # this allow to run some commands that requires libuterm to
+    # be injected in current process, as: wall
+    from atexit import register as at_exit_call
+    from ctypes import cdll
+    libutempter = cdll.LoadLibrary('libutempter.so.0')
+    if libutempter is not None:
+        # We absolutely need to remove the old tty from the utmp !!!
+        at_exit_call(libutempter.utempter_remove_added_record)
+except Exception as e:
+    libutempter = None
+    sys.stderr.write("[WARN] ===================================================================\n")
+    sys.stderr.write("[WARN] Unable to load the library libutempter !\n")
+    sys.stderr.write(
+        "[WARN] Some feature might not work:\n"
+        "[WARN]  - 'exit' command might freeze the terminal instead of closing the tab\n"
+        "[WARN]  - the 'wall' command is know to work badly\n"
+    )
+    sys.stderr.write("[WARN] Error: " + str(e) + '\n')
+    sys.stderr.write(
+        "[WARN] ===================================================================²\n"
+    )
+
 
 def halt(loc):
     code.interact(local=loc)
 
 
-# TODO PORT
-# __all__ = ['Terminal', 'TerminalBox']
+__all__ = ['GuakeTerminal']
 
 # pylint: enable=anomalous-backslash-in-string
 
 
-class TerminalBox(Gtk.HBox):
-
-    """A box to group the terminal and a scrollbar.
-    """
-
-    def __init__(self):
-        super(TerminalBox, self).__init__()
-        self.terminal = Terminal()
-        self.add_terminal()
-        self.add_scrollbar()
-
-    def add_terminal(self):
-        """Packs the terminal widget.
-        """
-        self.pack_start(self.terminal, True, True, 0)
-        self.terminal.show()
-
-    def add_scrollbar(self):
-        """Packs the scrollbar.
-        """
-        adj = self.terminal.get_vadjustment()
-        scroll = Gtk.VScrollbar.new(adj)
-        scroll.set_no_show_all(True)
-        self.pack_start(scroll, False, False, 0)
-
-
 class GuakeTerminal(Vte.Terminal):
 
     """Just a vte.Terminal with some properties already set.
     """
 
-    def __init__(self, window, settings):
+    def __init__(self, guake):
         super(GuakeTerminal, self).__init__()
-        self.window = window
-        self.settings = settings
+        self.guake = guake
         self.configure_terminal()
         self.add_matches()
         self.connect('button-press-event', self.button_press)
+        self.connect('button-press-event', self.button_press)
+        self.connect('child-exited', self.on_child_exited)
         self.matched_value = ''
         self.font_scale_index = 0
         self._pid = None
@@ -130,39 +129,42 @@ class GuakeTerminal(Vte.Terminal):
         else:
             super().feed_child(resolved_cmdline, len(resolved_cmdline))
 
+    def execute_command(self, command):
+        if command[-1] != '\n':
+            command += "\n"
+        self.feed_child(command)
+
     def copy_clipboard(self):
         if self.get_has_selection():
             super(GuakeTerminal, self).copy_clipboard()
         elif self.matched_value:
-            guake_clipboard = Gtk.Clipboard.get_default(self.window.get_display())
-            guake_clipboard.set_text(self.matched_value)
+            guake_clipboard = Gtk.Clipboard.get_default(self.guake.window.get_display())
+            guake_clipboard.set_text(self.matched_value, len(self.matched_value))
 
     def configure_terminal(self):
         """Sets all customized properties on the terminal
         """
-        client = self.settings.general
+        client = self.guake.settings.general
         word_chars = client.get_string('word-chars')
         if word_chars:
-            # TODO PORT this does not work this way any more see:
-            #   https://lazka.github.io/
-            #          pgi-docs/Vte-2.91/classes/Terminal.html#Vte.Terminal.set_word_char_exceptions
-            # self.set_word_chars(word_chars)
-            pass
+            self.set_word_char_exceptions(word_chars)
         self.set_audible_bell(client.get_boolean('use-audible-bell'))
         self.set_sensitive(True)
 
-        cursor_blink_mode = self.settings.style.get_int('cursor-blink-mode')
+        cursor_blink_mode = self.guake.settings.style.get_int('cursor-blink-mode')
         self.set_property('cursor-blink-mode', cursor_blink_mode)
 
         if (Vte.MAJOR_VERSION, Vte.MINOR_VERSION) >= (0, 50):
             self.set_allow_hyperlink(True)
 
-        # TODO PORT there is no method set_flags anymore
-        # self.set_flags(gtk.CAN_DEFAULT)
-        # self.set_flags(gtk.CAN_FOCUS)
-        # TODO PORT getting it and then setting it???
-        # cursor_shape = client.get_int(KEY('/style/cursor_shape'))
-        # client.set_int(KEY('/style/cursor_shape'), cursor_shape)
+        # TODO PORT is this still the case with the newer vte version?
+        # -- Ubuntu has a patch to libvte which disables mouse scrolling in apps
+        # -- like vim and less by default. If this is the case, enable it back.
+        if hasattr(self, "set_alternate_screen_scroll"):
+            self.set_alternate_screen_scroll(True)
+
+        self.set_can_default(True)
+        self.set_can_focus(True)
 
     def add_matches(self):
         """Adds all regular expressions declared in
@@ -260,19 +262,22 @@ class GuakeTerminal(Vte.Terminal):
 
         pt = Path(text)
         log.debug("checking file existance: %r", pt)
-        if pt.exists():
-            lineno = find_lineno(text, pt, lineno, py_func)
-            log.info("File exists: %r, line=%r", pt.absolute().as_posix(), lineno)
-            return (pt, lineno, colno)
-        log.debug("No file found matching: %r", text)
-        cwd = self.get_current_directory()
-        pt = Path(cwd) / pt
-        log.debug("checking file existance: %r", pt)
-        if pt.exists():
-            lineno = find_lineno(text, pt, lineno, py_func)
-            log.info("File exists: %r, line=%r", pt.absolute().as_posix(), lineno)
-            return (pt, lineno, colno)
-        log.info("file does not exist: %s", str(pt))
+        try:
+            if pt.exists():
+                lineno = find_lineno(text, pt, lineno, py_func)
+                log.info("File exists: %r, line=%r", pt.absolute().as_posix(), lineno)
+                return (pt, lineno, colno)
+            log.debug("No file found matching: %r", text)
+            cwd = self.get_current_directory()
+            pt = Path(cwd) / pt
+            log.debug("checking file existance: %r", pt)
+            if pt.exists():
+                lineno = find_lineno(text, pt, lineno, py_func)
+                log.info("File exists: %r, line=%r", pt.absolute().as_posix(), lineno)
+                return (pt, lineno, colno)
+            log.debug("file does not exist: %s", str(pt))
+        except OSError:
+            log.debug("not a file name: %r", text)
         return (None, None, None)
 
     def button_press(self, terminal, event):
@@ -305,6 +310,11 @@ class GuakeTerminal(Vte.Terminal):
             self.found_link = self.handleTerminalMatch(matched_string)
             self.matched_value = matched_string[0]
 
+    def on_child_exited(self, target, status, *user_data):
+        if libutempter is not None:
+            if self.get_pty() is not None:
+                libutempter.utempter_remove_record(self.get_pty().get_fd())
+
     def quick_open(self):
         self.copy_clipboard()
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
@@ -319,7 +329,7 @@ class GuakeTerminal(Vte.Terminal):
         found_matcher = False
         log.debug("matched string: %s", matched_string)
         # First searching in additional matchers
-        use_quick_open = self.settings.general.get_boolean("quick-open-enable")
+        use_quick_open = self.guake.settings.general.get_boolean("quick-open-enable")
         if use_quick_open:
             found_matcher = self._find_quick_matcher(value)
         if not found_matcher:
@@ -351,7 +361,7 @@ class GuakeTerminal(Vte.Terminal):
     def _execute_quick_open(self, filepath, line_number):
         if not filepath:
             return
-        cmdline = self.settings.general.get_string("quick-open-command-line")
+        cmdline = self.guake.settings.general.get_string("quick-open-command-line")
         if not line_number:
             line_number = ""
         else:
@@ -359,7 +369,7 @@ class GuakeTerminal(Vte.Terminal):
         logging.debug("Opening file %s at line %s", filepath, line_number)
         resolved_cmdline = cmdline % {"file_path": filepath, "line_number": line_number}
         logging.debug("Command line: %s", resolved_cmdline)
-        quick_open_in_current_terminal = self.settings.general.get_boolean(
+        quick_open_in_current_terminal = self.guake.settings.general.get_boolean(
             "quick-open-in-current-terminal"
         )
         if quick_open_in_current_terminal:
@@ -392,7 +402,11 @@ class GuakeTerminal(Vte.Terminal):
         if value:
             return value
 
+    def get_link_under_cursor(self):
+        return self.found_link
+
     def browse_link_under_cursor(self):
+        # TODO move the call to xdg-open to guake.utils
         if not self.found_link:
             return
         log.debug("Opening link: %s", self.found_link)
@@ -464,31 +478,36 @@ class GuakeTerminal(Vte.Terminal):
                 # did the work and SIGKILL wasnt needed.
                 pass
 
+    def spawn_sync_pid(self, directory):
 
-# TODO PORT port this from HBOX to Box with orientation horitzontal
+        argv = list()
+        user_shell = self.guake.settings.general.get_string('default-shell')
+        if user_shell and os.path.exists(user_shell):
+            argv.append(user_shell)
+        else:
+            argv.append(os.environ['SHELL'])
 
+        login_shell = self.guake.settings.general.get_boolean('use-login-shell')
+        if login_shell:
+            argv.append('--login')
 
-class GuakeTerminalBox(Gtk.HBox):
+        log.debug("Spawn command: \"%s\"", " ".join(argv))
 
-    """A box to group the terminal and a scrollbar.
-    """
+        pid = self.spawn_sync(
+            Vte.PtyFlags.DEFAULT, directory, argv, [], GLib.SpawnFlags.DO_NOT_REAP_CHILD, None,
+            None, None
+        )
+        try:
+            tuple_type = gi._gi.ResultTuple  # pylint: disable=c-extension-no-member
+        except:  # pylint: disable=bare-except
+            tuple_type = tuple
+        if isinstance(pid, (tuple, tuple_type)):
+            # Return a tuple in 2.91
+            # https://lazka.github.io/pgi-docs/Vte-2.91/classes/Terminal.html#Vte.Terminal.spawn_sync
+            pid = pid[1]
+        assert isinstance(pid, int)
 
-    def __init__(self, window, settings):
-        super(GuakeTerminalBox, self).__init__()
-        self.terminal = GuakeTerminal(window, settings)
-        self.add_terminal()
-        self.add_scroll_bar()
-
-    def add_terminal(self):
-        """Packs the terminal widget.
-        """
-        self.pack_start(self.terminal, True, True, 0)
-        self.terminal.show()
-
-    def add_scroll_bar(self):
-        """Packs the scrollbar.
-        """
-        adj = self.terminal.get_vadjustment()
-        scroll = Gtk.VScrollbar(adj)
-        scroll.show()
-        self.pack_start(scroll, False, False, 0)
+        if libutempter is not None:
+            libutempter.utempter_add_record(self.get_pty().get_fd(), os.uname()[1])
+        self.pid = pid
+        return pid
